# This R script could be used to extract CSREs in chromosome 1
# You have to get the binary matrices prepared and saved as "H1.chr1.RData" (cell type name + chromosome)
library(wavelets)
print("Loading data ... ...")
filenames <- dir()
filenames <- filenames[grep(".chr1.RData", filenames)]
cell.type.names <- sub(".chr1.RData$", '', filenames)
binary.matrices <- list()
chromosome <- "chr1"
for (iter in filenames) {
  load(iter)
  eval(parse(text = paste("temp <- ", sub(".chr1.RData$", '', iter))))
  rm(list = sub(".chr1.RData$", '', iter))
  binary.matrices[[length(binary.matrices) + 1]] <- temp
}
print("Computing hamming matrix ... ...")
cell.type.number <- length(binary.matrices)
hamming <- array(0, dim = c(cell.type.number, ncol(binary.matrices[[1]])))
for (i in 1:(cell.type.number - 1)) {
  for (j in (i + 1):cell.type.number) {
    temp <- apply(binary.matrices[[i]] != binary.matrices[[j]], 2, sum)
	hamming[i, ] <- hamming[i, ] + temp
    hamming[j, ] <- hamming[j, ] + temp
  }
}
# Normalization and Correction
print("Normalization and correction ... ...")
NORMALIZATION.constant <- 1000
for (iter in 1:cell.type.number) {
	hamming[iter, ] <- hamming[iter, ] * NORMALIZATION.constant / norm(hamming[iter, , drop = FALSE], type = "F")
}
mu <- apply(hamming, 2, mean)
mu <- rep(mu, each = length(cell.type.names))
dim(mu) <- dim(hamming)
sigma <- apply(hamming, 2, sd) + 1e-5
sigma <- rep(sigma, each = length(cell.type.names))
dim(sigma) <- dim(hamming)
hamming <- (hamming - mu)/sigma
# MRA - wavelet smoothing
print("wavelet smoothing ... ...")
mra <- array(0, dim = dim(hamming))
for (iter in 1:length(cell.type.names)) {
  temp <- hamming[iter, ]
  temp.mra <- mra(temp, n.levels = 7, fast = TRUE, method = "modwt")
  mra[iter, ] <- temp.mra@S[[3]]
}
# Extracting CSREs, you can modify the following two lines to change the height and length paramters
height.parameter <- 0.5
length.parameter <- 12
for (iter in c(1:length(cell.type.names))) {
  v <- mra[iter, ]
  v[v <= region.height] <- 0
  v[v > region.height] <- 1
  v1 <- c(0, v)
  v2 <- c(v, 0)
  idx <- which((v1 + v2) == 1)
  if (length(idx) == 0) {
    next
  }
  region.start <- idx[seq(from = 1, to = length(idx) - 1, by = 2)]
  region.end <- idx[seq(from = 2, to = length(idx), by = 2)] - 1
  region.length <- region.end - region.start + 1
  idx <- which(region.length >= region.width)
  region.start <- region.start[idx]
  region.end <- region.end[idx]
  region.length <- region.length[idx]
  ### Output
  temp <- array(0, dim = c(length(region.start), 4))
  temp[, 1] <- chromosome
  temp[, 2] <- region.start
  temp[, 3] <- region.end
  temp[, 4] <- region.length
  write(t(temp), file = paste(cell.type.names[iter], "CSRE", sep = "."), ncolumns = 4, append = TRUE, sep = " ")
}